<?php

/*

 Plugin Name: BestsellerAzon

 Plugin URI: http://bestsellerazon.com

 Description: Quickly and easily add the bestselling items from any category on Amazon into your content or sidebars

 Version: 1.1.5

 Author: BestsellerAzon

 Author URI: http://bestsellerazon.com/

 */



if (!class_exists('BestsellerAzon')) {

	class BestsellerAzon {

		/// CONSTANTS



		//// VERSION

		const VERSION = '1.1.5';



		//// IDS

		const SERVICE_PRODUCT_ID = '9';

		const SERVICE_VERIFICATION_URL = 'http://bestsellerazon.com';



		//// KEYS

		const POST_META_KEY = '_bestsellerazon_post_meta';

		const SETTINGS_KEY = '_bestsellerazon_settings';

		const BESTSELLER_TRANSIENT_BASE_KEY = '_bsabs_%s_%s';

		const BESTSELLER_UPDATE_TIME_TRANSIENT_BASE_KEY = '_bsabst_%s_%s';



		// TRANSIENTS

		const TRANSIENT_VERIFICATION_CHECK = '_bestsellerazon_by_key';

		const TRANSIENT_VERIFICATION_CHECK_TIMEOUT = 600;

		// 10 Minutes



		//// SLUGS

		const SETTINGS_PAGE_SLUG = 'bestsellerazon';



		//// SHORTCODES

		const SHORTCODE = 'bestsellerazon';



		//// CACHE - 24 Hours

		const CACHE_PERIOD = 86400;



		/// DATA STORAGE

		private static $admin_page_hooks = array('post.php', 'post-new.php', 'widgets.php');

		public static $default_meta = array('headline' => 'default', 'headline-text' => '', 'enable' => 'default', 'display-prices' => 'default', 'display-descriptions' => 'default', 'category' => 'default', 'category-name' => 'Site Default', 'number' => 'default');

		private static $default_settings = array('accept-tos' => 'no', 'locale' => 'us', 'post-types' => array('post' => 'yes', 'page' => 'yes'), 'default-headline' => '', 'display-prices' => 'yes', 'display-descriptions' => 'no', 'enabled-by-default' => 'yes', 'number-bestsellers' => 10, 'default-category' => '1000', 'links-new-window' => 'yes', 'links-nofollow' => 'yes', 'links-cloaking-prefix' => 'bestseller', 'display-contexts' => array('single' => 'yes',));



		public static function init() {

			self::add_actions();

			self::add_filters();

			self::initialize_defaults();

			self::register_shortcodes();



			self::initialize_api();

		}



		private static function add_actions() {

			if (is_admin()) {

				add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_administrative_resources'));

				add_action('admin_menu', array(__CLASS__, 'add_administrative_interface_items'));

				add_action('add_meta_boxes', array(__CLASS__, 'add_meta_boxes'), 10, 2);

				add_action('save_post', array(__CLASS__, 'save_post_meta'), 10, 2);

			} else {

				add_action('parse_request', array(__CLASS__, 'redirect_to_amazon'));

				add_action('wp_head', array(__CLASS__, 'enqueue_frontend_resources'), 1);

			}



			add_action('widgets_init', array(__CLASS__, 'register_widget'));

			add_action('wp_ajax_bestsellerazon_get_browse_node_options', array(__CLASS__, 'ajax_output_browse_node_options'));

		}



		private static function add_filters() {

			add_filter('bestsellerazon_get_settings', array(__CLASS__, 'add_default_category'));

			add_filter('bestsellerazon_pre_meta_save', array(__CLASS__, 'sanitize_meta'));

			add_filter('bestsellerazon_pre_settings_save', array(__CLASS__, 'sanitize_settings'), 10, 2);

			add_filter('generate_rewrite_rules', array(__CLASS__, 'add_rewrite_rules'));

			add_filter('plugin_action_links_' . plugin_basename(__FILE__), array(__CLASS__, 'add_settings_link'));

			add_filter('query_vars', array(__CLASS__, 'add_query_vars'));

			add_filter('the_content', array(__CLASS__, 'add_amazon_bestsellers'));

		}



		private static function initialize_api() {

			$settings = self::get_settings();



			BS_Amazon_API::set_credentials($settings['access-key-id'], $settings['secret-access-key']);

		}



		private static function initialize_defaults() {

			self::$default_settings['default-headline'] = __('Bestselling Products on Amazon');



			$settings = self::get_settings();

			self::$default_meta['headline-text'] = $settings['default-headline'];

		}



		private static function register_shortcodes() {

			add_shortcode(self::SHORTCODE, array(__CLASS__, 'display_bestsellers'));

		}



		/// AJAX CALLBACKS



		public static function ajax_output_browse_node_options() {

			$data = stripslashes_deep($_REQUEST);



			$browse_node_id = $data['browse_node_id'];

			$locale = $data['locale'];

			$hierarchy = array_map('trim', explode(',', $data['hierarchy']));



			$browse_nodes = self::get_browse_nodes($browse_node_id, $locale, $hierarchy);



			foreach ($browse_nodes as $browse_node_id => $browse_node) {

				printf('<option data-ancestral="%s" data-hierarchy="%s" data-children="%d" value="%s">%s</option>', esc_attr($browse_node['ancestral_name']), implode(',', $browse_node['hierarchy']), $browse_node['has_children'], esc_attr($browse_node_id), esc_html($browse_node['name']));

			}



			exit;

		}



		/// CALLBACKS



		public static function add_amazon_bestsellers($content) {

			$settings = self::get_settings();

			

			$license_validated = !is_wp_error(self::validate_clickbank_receipt($settings['clickbank-receipt-id']));

			$accept_tos = 'yes' == $settings['accept-tos'];

			$should_display = false !== self::should_display_bestsellers(get_the_ID());



			$home = is_home() && 'yes' === $settings['display-contexts']['home'];

			$tax = (is_category() || is_tag()) && 'yes' === $settings['display-contexts']['term'];

			$singular = is_singular(array_keys($settings['post-types'])) && 'yes' === $settings['display-contexts']['single'];



			if ($license_validated && $accept_tos && $should_display && ($home || $tax || $singular)) {

				$content .= self::display_bestsellers(array('category' => self::get_browse_node_id(get_the_ID()), 'display_descriptions' => self::get_display_descriptions(get_the_ID()), 'display_prices' => self::get_display_prices(get_the_ID()), 'headline' => self::get_headline(get_the_ID()), 'locale' => self::get_locale(), 'number' => self::get_number(get_the_ID())));

			}



			return $content;

		}



		public static function add_administrative_interface_items() {

			self::$admin_page_hooks[] = $settings = add_options_page(__('BestsellerAzon Settings'), __('BestsellerAzon'), 'manage_options', self::SETTINGS_PAGE_SLUG, array(__CLASS__, 'display_settings_page'));



			add_action("load-{$settings}", array(__CLASS__, 'process_settings_save'));

		}



		public static function add_default_category($settings) {

			if (empty($settings['default-category']) || empty($settings['default-category-name'])) {

				$settings['default-category'] = BS_Amazon_API::get_locale_default_browse_node_id($settings['locale']);

				$settings['default-category-name'] = BS_Amazon_API::get_locale_default_browse_node_name($settings['locale']);

			}



			return $settings;

		}



		public static function add_meta_boxes($post_type, $post) {

			$settings = self::get_settings();



			if ('yes' == $settings['post-types'][$post_type] && 'yes' == $settings['accept-tos'] && !is_wp_error(self::validate_clickbank_receipt($settings['clickbank-receipt-id']))) {

				add_meta_box('bestsellerazon-meta-box', __('BestsellerAzon'), array(__CLASS__, 'display_meta_box'), $post_type, 'normal');

			}

		}



		public static function add_query_vars($vars) {

			$vars[] = 'bestsellerazon-asin';

			$vars[] = 'bestsellerazon-code';



			return $vars;

		}



		public static function add_rewrite_rules($wp_rewrite) {

			$settings = self::get_settings();



			if ('yes' == $settings['links-cloaking']) {

				$prefix = empty($settings['links-cloaking-prefix']) ? 'bestseller' : $settings['links-cloaking-prefix'];



				$new = array("{$prefix}/([^/]+)/([^/]+)/?$" => 'index.php?bestsellerazon-asin=' . $wp_rewrite->preg_index(1) . '&bestsellerazon-code=' . $wp_rewrite->preg_index(2));



				$wp_rewrite->rules = $new + $wp_rewrite->rules;

			}

		}



		public static function add_settings_link($links) {

			$settings_link = sprintf('<a href="%s">%s</a>', add_query_arg(array('page' => self::SETTINGS_PAGE_SLUG), admin_url('options-general.php')), __('Settings'));



			return array('settings' => $settings_link) + $links;

		}



		public static function enqueue_administrative_resources($hook) {

			if (!in_array($hook, self::$admin_page_hooks)) {

				return;

			}



			wp_enqueue_script('bestsellerazon-backend', plugins_url('resources/backend/bestsellerazon.js', __FILE__), array('jquery'), self::VERSION);

			wp_enqueue_style('bestsellerazon-backend', plugins_url('resources/backend/bestsellerazon.css', __FILE__), array(), self::VERSION);

		}



		public static function enqueue_frontend_resources() {

			wp_enqueue_style('bestsellerazon-frontend', plugins_url('resources/frontend/bestsellerazon.css', __FILE__), array(), self::VERSION);

		}



		public static function process_settings_save() {

			$data = stripslashes_deep($_POST);



			if (!empty($data['save-bestsellerazon-settings-nonce']) && wp_verify_nonce($data['save-bestsellerazon-settings-nonce'], 'save-bestsellerazon-settings')) {



				add_settings_error('general', 'settings_updated', __('Settings saved.'), 'updated');

				set_transient('settings_errors', get_settings_errors(), 30);



				self::set_settings(apply_filters('bestsellerazon_pre_settings_save', $data['bestsellerazon'], self::get_settings()));

				flush_rewrite_rules();



				wp_redirect(add_query_arg(array('page' => self::SETTINGS_PAGE_SLUG, 'settings-updated' => 'true'), admin_url('options-general.php')));

				exit ;

			}

		}



		public static function redirect_to_amazon($wp) {

			$vars = $wp->query_vars;



			if ($vars['bestsellerazon-asin'] && $vars['bestsellerazon-code']) {

				$settings = self::get_settings();



				$asin = $vars['bestsellerazon-asin'];

				$associate_tag = $vars['bestsellerazon-code'];

				$locale = $settings['locale'];



				$url = sprintf('http://www.amazon.%s/dp/%s', BS_Amazon_API::get_locale_tld($locale), $asin);

				if (!empty($associate_tag) && 'X' != $associate_tag) {

					$url = add_query_arg(array('tag' => $associate_tag), $url);

				}



				wp_redirect($url);

				exit ;

			}

		}



		public static function register_widget() {

			$settings = self::get_settings();



			$license_validated = !is_wp_error(self::validate_clickbank_receipt($settings['clickbank-receipt-id']));

			$accept_tos = 'yes' == $settings['accept-tos'];



			if($license_validated && $accept_tos) {

				register_widget('BestsellerazonWidget');

			}

		}



		public static function sanitize_meta($meta) {

			$meta = bestsellerazon_trim_r($meta);



			$meta['headline-text'] = strip_tags($meta['headline-text']);



			$meta['headline'] = $meta['headline'] == 'default' ? 'default' : 'other';



			if (!in_array($meta['enable'], array('default', 'yes', 'no'))) {

				$meta['enable'] = 'default';

			}



			if (!in_array($meta['display-prices'], array('default', 'yes', 'no'))) {

				$meta['display-prices'] = 'default';

			}

			

			if (!in_array($meta['display-descriptions'], array('default', 'yes', 'no'))) {

				$meta['display-descriptions'] = 'default';

			}



			if ('default' != $meta['number'] && !in_array($meta['number'], range(1, 10))) {

				$meta['number'] = 'default';

			}



			$settings = self::get_settings();

			if (!empty($meta['category'])) {

				$meta['category-name'] = self::get_browse_node_name($meta['category'], $settings['locale']);

			}



			return $meta;

		}



		public static function sanitize_settings($settings, $old_settings) {

			$settings = bestsellerazon_trim_r($settings);



			$settings['accept-tos'] = 'yes' == $settings['accept-tos'] ? 'yes' : 'no';



			if ('yes' === $settings['has-licensed']) {

				// Amazon

				if (!in_array($settings['locale'], BS_Amazon_API::get_locale_keys())) {

					$settings['locale'] = 'us';

				}



				// Display



				if (!is_array($settings['post-types'])) {

					$settings['post-types'] = array();

				}



				if (!in_array($settings['number-bestsellers'], range(1, 10))) {

					$settings['number-bestsellers'] = 10;

				}



				foreach (array('enabled-by-default', 'links-new-window', 'links-nofollow', 'links-cloaking', 'display-prices', 'display-descriptions') as $checkbox_key) {

					$settings[$checkbox_key] = 'yes' == $settings[$checkbox_key] ? 'yes' : 'no';

				}



				$display_contexts = array();

				foreach(array('single', 'term', 'home') as $display_context_key) {

					$display_contexts[$display_context_key] = 'yes' == $settings['display-contexts'][$display_context_key] ? 'yes' : 'no';

				}

				$settings['display-contexts'] = $display_contexts;



				if (empty($settings['links-cloaking-prefix'])) {

					$settings['links-cloaking-prefix'] = self::$default_settings['links-cloaking-prefix'];

				}



				$settings['default-headline'] = strip_tags($settings['default-headline']);



				if(empty($settings['default-category'])) {

					$settings['default-category'] = BS_Amazon_API::get_locale_default_browse_node_id($settings['locale']);

				}



				if (!empty($settings['default-category'])) {

					$settings['default-category-name'] = self::get_browse_node_name($settings['default-category'], $settings['locale']);

				}

			} else {

				$settings = array_merge($old_settings, $settings);

			}



			return $settings;

		}



		public static function save_post_meta($post_id, $post) {

			$data = stripslashes_deep($_POST);

			if (wp_is_post_autosave($post_id) || wp_is_post_revision($post_id) || !wp_verify_nonce($data['save-bestsellerazon-meta-nonce'], 'save-bestsellerazon-meta')) {

				return;

			}



			self::set_meta($post_id, apply_filters('bestsellerazon_pre_meta_save', $data['bestsellerazon']));

		}



		/// DISPLAY CALLBACKS



		public static function display_category_selector($category_name, $category_id, $category_id_input_id, $category_id_input_name, $access_key_id, $secret_access_key, $allow_default = false, $hide_help = false) {

			include ('views/backend/misc/category-selector.php');

		}



		public static function display_meta_box($post) {

			$meta = self::get_meta($post->ID);

			$settings = self::get_settings();



			include ('views/backend/meta-boxes/meta-box.php');

		}



		public static function display_settings_page() {

			$settings = self::get_settings();



			add_meta_box('bestsellerazon-settings-licensing', __('Licensing'), array(__CLASS__, 'display_settings_meta_box_licensing'), 'bestsellerazon-settings', 'normal');



			$has_licensed = true;

			$validation = self::validate_clickbank_receipt($settings['clickbank-receipt-id']);

			if (!is_wp_error($validation) && 'yes' === $settings['accept-tos']) {

				add_meta_box('bestsellerazon-settings-api', __('API Credentials'), array(__CLASS__, 'display_settings_meta_box_api'), 'bestsellerazon-settings', 'normal');

				add_meta_box('bestsellerazon-settings-locale', __('Amazon Options'), array(__CLASS__, 'display_settings_meta_box_locale'), 'bestsellerazon-settings', 'normal');

				add_meta_box('bestsellerazon-settings-display', __('Display and Defaults'), array(__CLASS__, 'display_settings_meta_box_display'), 'bestsellerazon-settings', 'normal');

				add_meta_box('bestsellerazon-settings-links', __('Affiliate Links'), array(__CLASS__, 'display_settings_meta_box_links'), 'bestsellerazon-settings', 'normal');

			} else {

				$has_licensed = false;



				if(is_wp_error($validation) && 'yes' != $settings['accept-tos']) {

					add_settings_error('clickbank-receipt-id', 'clickbank-receipt-id-invalid', __('Please enter a valid license key and accept the BestsellerAzon Terms of Service to access the rest of the settings.'));

				} else if(is_wp_error($validation)) {

					add_settings_error('clickbank-receipt-id', 'clickbank-receipt-id-invalid', __('Please enter a valid license key to access the rest of the settings.'));

				} else {

					add_settings_error('clickbank-receipt-id', 'clickbank-receipt-id-invalid', __('Please accept the BestsellerAzon Terms of Service to access the rest of the settings.'));

				}

			}



			include ('views/backend/settings/settings.php');

		}



		//// SETTINGS META BOXES



		public static function display_settings_meta_box_licensing($settings) {

			include ('views/backend/settings/sections/01-licensing.php');

		}



		public static function display_settings_meta_box_api($settings) {

			include ('views/backend/settings/sections/02-api.php');

		}



		public static function display_settings_meta_box_locale($settings) {

			include ('views/backend/settings/sections/03-locale.php');

		}



		public static function display_settings_meta_box_display($settings) {

			include ('views/backend/settings/sections/04-display.php');

		}



		public static function display_settings_meta_box_links($settings) {

			$has_permalinks = '' != get_option('permalink_structure');



			include ('views/backend/settings/sections/05-links.php');

		}



		/// SHORTCODE CALLBACKS



		public static function display_bestsellers($atts = array(), $content = null) {

			$atts = shortcode_atts(array('category' => 0, 'display_descriptions' => false, 'display_prices' => true, 'locale' => 'us', 'number' => 10, 'headline' => ''), $atts);

			extract($atts);



			if (empty($category)) {

				return '';

			}



			if (!in_array($locale, BS_Amazon_API::get_locale_keys())) {

				$locale = 'us';

			}



			if (!in_array($number, range(1, 10))) {

				$number = 10;

			}



			$bestseller_items = self::get_bestsellers($category, $locale);

			$last_updated_timestamp = self::get_bestsellers_update_time($category, $locale);



			ob_start();

			if (is_array($bestseller_items)) {

				$bestseller_items = array_slice($bestseller_items, 0, $number);

				$link_atts = self::get_link_atts();

				$resource_directory_url = plugins_url('resources/frontend', __FILE__);

				

				include ('views/frontend/amazon-items.php');

			}

			return ob_get_clean();

		}



		/// POST META



		private static function get_meta($post_id) {

			if (empty($post_id)) {

				global $post;

				$post_id = $post->ID;

			}



			$meta = wp_cache_get(self::POST_META_KEY, $post_id);



			if (false === $meta) {

				$meta = wp_parse_args((array)get_post_meta($post_id, self::POST_META_KEY, true), self::$default_meta);

				wp_cache_set(self::POST_META_KEY, $meta, $post_id, time() + CACHE_PERIOD);

			}



			return $meta;

		}



		private static function set_meta($post_id, $meta) {

			if (empty($post_id)) {

				global $post;

				$post_id = $post->ID;

			}



			$meta = wp_parse_args($meta, self::$default_meta);

			update_post_meta($post_id, self::POST_META_KEY, $meta);

			wp_cache_set(self::POST_META_KEY, $meta, $post_id, time() + CACHE_PERIOD);

		}



		/// SETTINGS



		public static function get_settings() {

			$settings = wp_cache_get(self::SETTINGS_KEY);



			if (!is_array($settings)) {

				$settings = apply_filters('bestsellerazon_get_settings', wp_parse_args(get_option(self::SETTINGS_KEY, self::$default_settings), self::$default_settings));

				wp_cache_set(self::SETTINGS_KEY, $settings, null, time() + CACHE_PERIOD);

			}



			return $settings;

		}



		private static function set_settings($settings) {

			if (is_array($settings)) {

				$settings = wp_parse_args($settings, self::$default_settings);

				update_option(self::SETTINGS_KEY, $settings);

				wp_cache_set(self::SETTINGS_KEY, $settings, null, time() + CACHE_PERIOD);

			}

		}



		/// BESTSELLERS



		private static function get_bestsellers_key($browse_node_id, $locale) {

			return sprintf(self::BESTSELLER_TRANSIENT_BASE_KEY, $browse_node_id, $locale);

		}



		public static function get_bestsellers($browse_node_id, $locale) {

			$bestsellers_key = self::get_bestsellers_key($browse_node_id, $locale);



			$bestseller_items = get_transient($bestsellers_key);

			

			if (false === $bestseller_items) {

				$bestseller_items = array();



				$settings = self::get_settings();

				$top_sellers_response = BS_Amazon_API::browse_node_lookup($browse_node_id, 'TopSellers', $settings['associate-tag'], $locale);

				if (!is_wp_error($top_sellers_response) && is_array($top_sellers_response['BrowseNodes']['BrowseNode']['TopSellers']['TopSeller'])) {

					$bestseller_items = array();

					foreach ($top_sellers_response['BrowseNodes']['BrowseNode']['TopSellers']['TopSeller'] as $top_seller) {

						$bestseller_id = $top_seller['ASIN'];

						$bestseller_item_response = BS_Amazon_API::item_lookup($bestseller_id, 'ASIN', $settings['associate-tag'], $locale);

						

						if(!is_wp_error($bestseller_item_response) && is_array($bestseller_item_response['Items']['Item'])) {

							$bestseller_items = array_merge($bestseller_items, $bestseller_item_response['Items']['Item']);

						}

					}



					self::set_bestsellers($browse_node_id, $locale, $bestseller_items);

					self::set_bestsellers_update_time($browse_node_id, $locale, current_time('timestamp'));

				}

			}



			return $bestseller_items;

		}



		private static function set_bestsellers($browse_node_id, $locale, $bestseller_items) {

			$bestsellers_key = self::get_bestsellers_key($browse_node_id, $locale);



			if (is_array($bestseller_items)) {

				set_transient($bestsellers_key, $bestseller_items, self::CACHE_PERIOD);

				wp_cache_set($bestsellers_key, $bestseller_items, null, time() + self::CACHE_PERIOD);

			}

		}



		private static function get_bestsellers_update_time_key($browse_node_id, $locale) {

			return sprintf(self::BESTSELLER_UPDATE_TIME_TRANSIENT_BASE_KEY, $browse_node_id, $locale);

		}



		public static function get_bestsellers_update_time($browse_node_id, $locale) {

			$bestsellers_update_time_key = self::get_bestsellers_update_time_key($browse_node_id, $locale);



			$time = get_transient($bestsellers_update_time_key);



			if(false === $time) {

				$time = current_time('timestamp');

				self::set_bestsellers_update_time($browse_node_id, $locale, $time);

			}



			return $time;

		}



		private static function set_bestsellers_update_time($browse_node_id, $locale, $time) {

			$bestsellers_update_time_key = self::get_bestsellers_update_time_key($browse_node_id, $locale);



			set_transient($bestsellers_update_time_key, $time, self::CACHE_PERIOD);

			wp_cache_set($bestsellers_update_time_key, $time, null, time() + self::CACHE_PERIOD);

		}



		/// UTILITY



		private static function get_browse_nodes($browse_node_id, $locale, $hierarchy = array(-1)) {

			$returned = array();



			$queried_browse_node_id = intval($browse_node_id);



			if (0 == $queried_browse_node_id) {

				array_unshift($hierarchy, $queried_browse_node_id);

				$browse_nodes = BS_Amazon_API::get_locale_browse_node_indexes($locale);

				foreach ($browse_nodes as $browse_node_id => $browse_node_name) {

					$returned[$browse_node_id] = array('name' => $browse_node_name, 'ancestral_name' => $browse_node_name, 'has_children' => true, 'hierarchy' => $hierarchy);

				}

			} else {

				$browse_node_info = BS_Amazon_API::browse_node_lookup($browse_node_id, 'BrowseNodeInfo', '', $locale);

				$browse_node_structure = $browse_node_info['BrowseNodes']['BrowseNode'];

				array_unshift($hierarchy, $browse_node_structure['BrowseNodeId']);

				

				if (is_array($browse_node_structure['Children']['BrowseNode'])) {

					if(isset($browse_node_structure['Children']['BrowseNode']['BrowseNodeId'])) {

						$browse_node_structure['Children']['BrowseNode'] = array($browse_node_structure['Children']['BrowseNode']);

					}



					foreach ($browse_node_structure['Children']['BrowseNode'] as $child) {

						$child_info = BS_Amazon_API::browse_node_lookup($child['BrowseNodeId'], 'BrowseNodeInfo', '', $locale);

						$child_browse_node_structure = $child_info['BrowseNodes']['BrowseNode'];



						if(is_array($child_browse_node_structure) && isset($child_browse_node_structure['IsCategoryRoot']) && 1 == $child_browse_node_structure['IsCategoryRoot']) {

							return self::get_browse_nodes($child_browse_node_structure['BrowseNodeId'], $locale, array_slice($hierarchy, 1));

						} else if(is_array($child_browse_node_structure) && isset($child_browse_node_structure['Name'])) {

							$return_structure = array(

								'ancestral_name' => self::get_browse_node_ancestral_name($child_browse_node_structure),

								'has_children' => is_array($child_browse_node_structure['Children']['BrowseNode']) && !empty($child_browse_node_structure['Children']['BrowseNode']), 

								'name' => $child_browse_node_structure['Name'],

								'hierarchy' => $hierarchy,

							);

							

							$returned[$child_browse_node_structure['BrowseNodeId']] = $return_structure;

						}

					}

				}

			}



			return $returned;

		}



		private static function get_browse_node_name($browse_node_id, $locale) {

			if ('default' == $browse_node_id) {

				$browse_node_name = __('Site Default');

			} else {

				$browse_node_info = BS_Amazon_API::browse_node_lookup($browse_node_id, '', '', $locale);

				$browse_node_structure = !is_wp_error($browse_node_info) ? $browse_node_info['BrowseNodes']['BrowseNode'] : false;





				if (is_array($browse_node_structure)) {

					$browse_node_name = self::get_browse_node_ancestral_name($browse_node_structure);

				}

			}



			return $browse_node_name;

		}

		

		private static function get_browse_node_ancestral_name($browse_node_structure) {

			$name_parts = array();

			

			do {

				$name_parts[] = $browse_node_structure['Name'];

				$browse_node_structure = $browse_node_structure['Ancestors']['BrowseNode'];

			} while(is_array($browse_node_structure));

			

			return implode(' > ', array_reverse($name_parts));

		}



		private static function get_browse_node_id($post_id) {

			$meta = self::get_meta($post_id);



			$browse_node_id = $meta['category'];

			if ('default' === $browse_node_id || empty($browse_node_id)) {

				$settings = self::get_settings();



				$browse_node_id = $settings['default-category'];

			}



			return $browse_node_id;

		}

		

		private static function get_display_descriptions($post_id) {

			$meta = self::get_meta($post_id);

			

			$display_descriptions = $meta['display-descriptions'];

			if('default' == $display_descriptions) {

				$settings = self::get_settings();

				

				$display_descriptions = $settings['display-descriptions'];

			}

			

			return 'yes' === $display_descriptions;

		}



		private static function get_display_prices($post_id) {

			$meta = self::get_meta($post_id);



			$display_prices = $meta['display-prices'];

			if('default' == $display_prices) {

				$settings = self::get_settings();



				$display_prices = $settings['display-prices'];

			}



			return 'yes' === $display_prices;

		}



		private static function get_headline($post_id) {

			$meta = self::get_meta($post_id);

			

			$headline = $meta['headline-text'];

			if('default' == $meta['headline']) {

				$settings = self::get_settings();

				

				$headline = $settings['default-headline'];

			}

			

			return $headline;

		}



		public static function get_link_atts() {

			$settings = self::get_settings();



			$link_atts = array();



			if ('yes' == $settings['links-new-window']) {

				$link_atts[] = 'target="_blank"';

			}



			if ('yes' == $settings['links-nofollow']) {

				$link_atts[] = 'rel="nofollow"';

			}



			return implode(' ', $link_atts);

		}



		private static function get_locale() {

			$settings = self::get_settings();



			return $settings['locale'];

		}



		private static function get_number($post_id) {

			$meta = self::get_meta($post_id);



			$number = $meta['number'];

			if ('default' == $number) {

				$settings = self::get_settings();



				$number = $settings['number'];

			}



			return $number;

		}



		private static function should_display_bestsellers($post_id) {

			$meta = self::get_meta($post_id);

			$settings = self::get_settings();

			$type = get_post_type($post_id);



			return 'yes' === $settings['post-types'][$type] && ('yes' === $meta['enable'] || ('default' === $meta['enable'] && 'yes' === $settings['enabled-by-default']));

		}



		/// TEMPLATE TAGS



		public static function get_detail_page_url($item) {

			$settings = self::get_settings();



			if ('yes' == $settings['links-cloaking']) {

				$associate_tag = empty($settings['associate-tag']) ? 'X' : $settings['associate-tag'];



				return sprintf('%s/%s/%s/%s/', home_url(''), $settings['links-cloaking-prefix'], $item['ASIN'], $associate_tag);

			} else {

				$detail_page_url = urldecode($item['DetailPageURL']);

				if (empty($settings['associate-tag'])) {

					$detail_page_url = remove_query_arg('tag', $detail_page_url);

				} else {

					$detail_page_url = add_query_arg(array('tag' => $settings['associate-tag']), $detail_page_url);

				}



				$detail_page_url = add_query_arg(array('linkCode' => 'as2'), $detail_page_url);



				return $detail_page_url;

			}

		}



		/// LICENSING



		private static function validate_clickbank_receipt($receipt_id) {

			if (defined('BESTSELLERAZON_DEV') && BESTSELLERAZON_DEV) {

				return true;

			}



			$receipt_id = trim($receipt_id);

			$transient_key = self::TRANSIENT_VERIFICATION_CHECK . "_{$receipt_id}";

			$response = get_transient($transient_key);



			if (empty($response) || (is_object($response) && (!isset($response->is_valid) || false == $response->is_valid))) {

				$request_url = add_query_arg(array('clickbank-receipt-id' => urlencode($receipt_id), 'clickbank-product-id' => self::SERVICE_PRODUCT_ID), self::SERVICE_VERIFICATION_URL);

				$response = wp_remote_get($request_url);

			}



			if (is_wp_error($response)) {

				$result = true;

			} else {

				$object = @json_decode(wp_remote_retrieve_body($response));



				if (is_object($object)) {

					set_transient($transient_key, $response, self::TRANSIENT_VERIFICATION_CHECK_TIMEOUT);

				}



				if (!is_object($object) || $object->is_valid) {

					$result = true;

				} else {

					$result = new WP_Error('invalid_clickbank_receipt_id', sprintf(__('Your ClickBank Receipt ID is invalid and plugin functionality has been disabled.  Please <a href="%s" target="_blank">enter a valid Receipt ID</a>.'), admin_url('options-general.php?page=easyazon')));

				}

			}



			return $result;

		}



	}



	require_once ('lib/bs-amazon-api.php');

	require_once ('lib/template-tags.php');

	require_once ('widget/widget.php');

	BestsellerAzon::init();

}

