<?php
class flybx_validation
{
	public $field;
	public $success;
	public $error_messages;
	
    public function __construct()
	{
		$this->field = array();
		$this->success = TRUE;
		$this->error_messages = array();
	}
	
	public function set_rule($name = '', $nice_name = '', $rules = '')
	{
		$name = trim($name);
		$rules = trim($rules);
		if (strlen($name) > 0)
		{
			$this->field[$name] = array('value' => '', 'nice_name' => $nice_name, 'rules' => array(), 'error_msg' => '');
			if (isset($_POST[$name])) $this->field[$name]['value'] = $_POST[$name];
			if (strlen($rules) > 0)
			{
				$rules_arr = explode('|', $rules);
				if (count($rules_arr) > 0)
				{
					foreach($rules_arr as $rule)
					{
						if (method_exists('flybx_validation', $rule) || function_exists($rule))
						{
							$this->field[$name]['rules'][] = $rule;
						}
					}
				}
			}
		}
	}
	
	public function run()
	{
		if (!isset($_POST) || count($_POST) == 0)
		{
			return;
		}
		foreach($this->field as $name => $field)
		{
			foreach($field['rules'] as $rule)
			{
				if (method_exists('flybx_validation', $rule))
				{
					$this->$rule($name);
				}
				elseif (function_exists($rule) && isset($_POST[$name]))
				{
					$_POST[$name] = $rule($field['value']);
				}
			}
		}
		return $this->success;
	}
	
	public function show_errors($prefix = '<div class="error">', $suffix = '</div>')
	{
		if ($this->success === FALSE)
		{
			echo $prefix.'<p><strong>'.implode('</strong></p><p><strong>', $this->error_messages).'</strong></p>'.$suffix;
		}
	}
	
	public function show_error($name = '', $prefix = '<span style="color: red;">', $suffix = '</span>')
	{
		if ($this->success === FALSE && $this->field[$name]['error_msg'] != '')
		{
			echo $prefix.$this->field[$name]['error_msg'].$suffix;
		}
	}
	
	public function set_value($name = '', $default = '')
	{
		if (isset($_POST[$name]))
		{
			return $_POST[$name];
		}
		else
		{
			return $default;
		}
	}
	
	public function set_select($name = '', $value = FALSE, $default = '')
	{
		if (isset($_POST[$name]) && $value !== FALSE && $_POST[$name] == $value)
		{
			return 'selected';
		}
		elseif(!isset($_POST[$name]) && $value == $default)
		{
			return 'selected';
		}
		return '';
	}
	
	public function set_checkbox($name = '', $value = FALSE, $default = '')
	{		
		if (isset($_POST[$name]) && is_array($_POST[$name]) && $value !== FALSE && in_array($value, $_POST[$name]))
		{
			return 'checked="checked"';
		}
		elseif ((isset($_POST[$name]) || isset($_POST['flybx_form_submitted'])) && $value !== FALSE && ((is_array($_POST[$name]) && !in_array($value, $_POST[$name])) || (!is_array($_POST[$name]) && !isset($_POST[$name]))))
		{
			return '';
		}
		elseif (isset($_POST[$name]) && !is_array($_POST[$name]) && $value !== FALSE)
		{
			if ($value == 'on' || $value == 'On')
			{
				return 'checked="checked"';
			}
			return '';
		}
		elseif(!isset($_POST[$name]) && $value == $default)
		{
			//setting default value when no submission was made
			return 'checked="checked"';
		}
		return '';
	}
	
	public function show_rules()
	{
		echo '<pre>';
		print_r($this->field);
		echo '</pre>';
	}
	
	public function debug()
	{
		echo '<pre>';
		print_r($this);
		echo '</pre>';
	}
	
	//custom validation rules
	public function required($name = '')
	{
		if (strlen(trim($this->field[$name]['value'])) == 0)
		{
			$this->success = FALSE;
			$this->error_messages[] = $this->field[$name]['nice_name'].' is required';
			$this->field[$name]['error_msg'] = $this->field[$name]['nice_name'].' is required';
		}
	}
	
	public function required_checkbox($name = '')
	{
		if (!is_array($this->field[$name]['value']) || count($this->field[$name]['value']) == 0)
		{
			$this->success = FALSE;
			$this->error_messages[] = $this->field[$name]['nice_name'].' is required';
			$this->field[$name]['error_msg'] = $this->field[$name]['nice_name'].' is required';
		}
	}
	
	public function is_numeric($name = '')
	{
		if (is_numeric(trim($this->field[$name]['value'])) === FALSE)
		{
			$this->success = FALSE;
			$this->error_messages[] = $this->field[$name]['nice_name'].' must be a number';
			$this->field[$name]['error_msg'] = $this->field[$name]['nice_name'].' must be a number';
		}
	}
	
	public function check_sticky($name = '')
	{
		if ($name == 'flybx_position' && $this->field['flybx_position']['value'] == '6' && $this->field['flybx_alignment']['value'] != 'left' && $this->field['flybx_alignment']['value'] != 'right')
		{
			$this->success = FALSE;
			$this->error_messages[] = 'Please set Alignment to either Left or Right when using "Sticky" template placement';
			$this->field['flybx_alignment']['error_msg'] = 'Please set Alignment to either Left or Right when using "Sticky" template placement';
		}
	}
	
}